<?php

// Error codes
define (ID3_OK, 0);
define (ID3_BAD_FILE, 1);
define (ID3_INVALID, 2);

class ID3Lib {

	public $tags; // The object in which we store all the tags
	public $title, $artist, $album, $genre, $track, $disc, $year; // Easy access to the generic tags
	private $_err; // Error code
	private $_errmsg = array (null, "Unable to open file", "Not a valid ID3 v2 header");
	
	// The constructor, reads the MP3 file and gets the tags
	public function __construct ($file)
	{
		
		// Open the file
		$file = fopen ($file, "rb");
		if (!$file) {
			$this->_err = ID3_BAD_FILE;
			return;
		}
		
		// Read in the header (first ten bytes)
		$fb = fread ($file, 10);
		
		// Check for valid ID3 signature
		if (substr ($fb, 0, 3) != "ID3") {
			$this->_err = ID3_INVALID;
			return;
		}
		
		// Get the information from the header
		$ver = $this->toInt(substr ($fb, 3, 2));
		$hFlags = $this->toInt($fb{5});
		$hSize = $this->calcSize (substr ($fb, 6));
		
		// Read in the entire ID3 head and parse all the frames
		$fb = fread ($file, $hSize); $i = 0;
		while ($i < $hSize) {
			
			// Read the frame header
			$id = substr ($fb, $i, 4);
			$size = $this->toInt (substr ($fb, $i + 4, 4));
			if (!$size || !$id)
				break;
			$flags = $this->toInt (substr ($fb, $i + 8, 2));
			$data = substr ($fb, $i + 10, $size);
			$i += 10 + $size;
			$this->tags->$id = $data;
			
		}
		
		// Save the common tags
		$this->title = $this->tags->TIT2;
		$this->album = $this->tags->TALB;
		$this->artist = $this->tags->TPE1;
		$this->track = $this->tags->TRCK;
		$this->year = $this->tags->TYER;
		$this->genre = $this->tags->TCON;
		$this->disc = $this->tags->TPOS;
		
		// Close the file
		fclose ($file);
		
	}
	
	// Saves APIC if present
	public function savePicture ($file)
	{
	
		// If APIC is present, get the JPEG data and save it
		if (isset ($this->tags->APIC)) {
			$f = fopen ($file, "wb");
			fwrite ($f, substr ($this->tags->APIC, strpos ($this->tags->APIC, 0xff)));
			fclose ($f);
			return true;
		}
		
		return false;
	
	}
	
	// Converts a string value to an integer
	private function toInt ($val) {
		$o = 0; $l = strlen ($val) - 1;
		for ($i = 0; $i < $l + 1; $i++) {
			$v = ord($val{$i});
			$o |= $v << (($l - $i) * 8);
		}
		return $o;
	}
	
	// Returns an error message based on any error that may have occured
	public function getErr () {
		return $this->_errmsg[$this->err];
	}
	
	// Takes a 16-bit MSB integer and returns it in LSB
	private function lsb16 ($num) {
		return ($num >> 8) | ($num << 8);
	}
	
	// Takes a 32-bit MSB integer and returns it in LSB
	private function lsb32 ($num) {
		return (($num & 0xff) << 24) | (($num & 0xff00) << 8) | (($num & 0xff0000) >> 8) | (($num & 0xff000000) >> 24);
	}

	// Calculates the size used in the ID3 header
	private function calcSize ($num) {
		$num = $this->toInt ($num);
		return ($num & 0x7f) | (($num & 0x7f00) >> 1) | (($num & 0x7f0000) >> 2) | (($num & 0x7f000000) >> 3);
	}
	
}

?>